@@ -1,7 +1,7 @@
 //
 //    This file is part of Dire Wolf, an amateur radio packet TNC.
 //
-//    Copyright (C) 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2019, 2020  John Langner, WB2OSZ
+//    Copyright (C) 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2019, 2020, 2021, 2023  John Langner, WB2OSZ
 //
 //    This program is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -33,7 +33,9 @@
  *			Internet Gateway (IGate)
  *			Ham Radio of Things - IoT with Ham Radio
  *			FX.25 Forward Error Correction.
- *		
+ *			IL2P Forward Error Correction.
+ *			Emergency Alert System (EAS) Specific Area Message Encoding (SAME) receiver.
+ *			AIS receiver for tracking ships.
  *
  *---------------------------------------------------------------*/
 
@@ -67,9 +69,8 @@
 #include <fcntl.h>
 #include <sys/types.h>
 #include <sys/ioctl.h>
-#ifdef __OpenBSD__
-#include <soundcard.h>
-#elif __APPLE__
+#if USE_SNDIO || __APPLE__
+// no need to include <soundcard.h>
 #else
 #include <sys/soundcard.h>
 #endif
@@ -124,7 +125,10 @@
 #include "ax25_link.h"
 #include "dtime_now.h"
 #include "fx25.h"
+#include "il2p.h"
 #include "dwsock.h"
+#include "dns_sd_dw.h"
+#include "dlq.h"		// for fec_type_t definition.
 
 
 //static int idx_decoded = 0;
@@ -135,7 +139,7 @@ static BOOL cleanup_win (int);
 static void cleanup_linux (int);
 #endif
 
-static void usage (char **argv);
+static void usage ();
 
 #if defined(__SSE__) && !defined(__APPLE__)
 
@@ -193,7 +197,6 @@ int main (int argc, char *argv[])
 	//int eof;
 	int j;
 	char config_file[100];
-	int xmit_calibrate_option = 0;
 	int enable_pseudo_terminal = 0;
 	struct digi_config_s digi_config;
 	struct cdigi_config_s cdigi_config;
@@ -223,13 +226,22 @@ int main (int argc, char *argv[])
 	int d_h_opt = 0;	/* "-d h" option for hamlib debugging.  Repeat for more detail */
 #endif
 	int d_x_opt = 1;	/* "-d x" option for FX.25.  Default minimal. Repeat for more detail.  -qx to silence. */
+	int d_2_opt = 0;	/* "-d 2" option for IL2P.  Default minimal. Repeat for more detail. */
 
-	int E_tx_opt = 0;		/* "-E n" Error rate % for clobbering trasmit frames. */
+	int aprstt_debug = 0;	/* "-d d" option for APRStt (think Dtmf) debug. */
+
+	int E_tx_opt = 0;		/* "-E n" Error rate % for clobbering transmit frames. */
 	int E_rx_opt = 0;		/* "-E Rn" Error rate % for clobbering receive frames. */
 
 	float e_recv_ber = 0.0;		/* Receive Bit Error Rate (BER). */
 	int X_fx25_xmit_enable = 0;	/* FX.25 transmit enable. */
 
+	int I_opt = -1;		/* IL2P transmit, normal polarity, arg is max_fec. */
+	int i_opt = -1;		/* IL2P transmit, inverted polarity, arg is max_fec. */
+
+	char x_opt_mode = ' ';		/* "-x N" option for transmitting calibration tones. */
+	int x_opt_chan = 0;		/* Split into 2 parts.  Mode e.g.  m, a, and optional channel. */
+
 	strlcpy(l_opt_logdir, "", sizeof(l_opt_logdir));
 	strlcpy(L_opt_logfile, "", sizeof(L_opt_logfile));
 	strlcpy(P_opt, "", sizeof(P_opt));
@@ -272,6 +284,8 @@ int main (int argc, char *argv[])
 				/* 1 = normal, 0 = no text colors. */
 				/* 2, 3, ... alternate escape sequences for different terminals. */
 
+// FIXME: consider case of no space between t and number.
+
 	for (j=1; j<argc-1; j++) {
 	  if (strcmp(argv[j], "-t") == 0) {
 	    t_opt = atoi (argv[j+1]);
@@ -287,12 +301,12 @@ int main (int argc, char *argv[])
 
 	text_color_init(t_opt);
 	text_color_set(DW_COLOR_INFO);
-	//dw_printf ("Dire Wolf version %d.%d (%s) Beta Test 4\n", MAJOR_VERSION, MINOR_VERSION, __DATE__);
+	//dw_printf ("Dire Wolf version %d.%d (%s) BETA TEST 7\n", MAJOR_VERSION, MINOR_VERSION, __DATE__);
 	//dw_printf ("Dire Wolf DEVELOPMENT version %d.%d %s (%s)\n", MAJOR_VERSION, MINOR_VERSION, "G", __DATE__);
 	dw_printf ("Dire Wolf version %d.%d\n", MAJOR_VERSION, MINOR_VERSION);
 
 
-#if defined(ENABLE_GPSD) || defined(USE_HAMLIB) || defined(USE_CM108)
+#if defined(ENABLE_GPSD) || defined(USE_HAMLIB) || defined(USE_CM108) || USE_AVAHI_CLIENT || USE_MACOS_DNSSD
 	dw_printf ("Includes optional support for: ");
 #if defined(ENABLE_GPSD)
 	dw_printf (" gpsd");
@@ -302,12 +316,16 @@ int main (int argc, char *argv[])
 #endif
 #if defined(USE_CM108)
 	dw_printf (" cm108-ptt");
+#endif
+#if (USE_AVAHI_CLIENT|USE_MACOS_DNSSD)
+	dw_printf (" dns-sd");
 #endif
 	dw_printf ("\n");
 #endif
 
 
 #if __WIN32__
+	//setlinebuf (stdout);   setvbuf???
 	SetConsoleCtrlHandler ((PHANDLER_ROUTINE)cleanup_win, TRUE);
 #else
 	setlinebuf (stdout);
@@ -357,7 +375,23 @@ int main (int argc, char *argv[])
 	text_color_set(DW_COLOR_INFO);
 #endif
 
+// I've seen many references to people running this as root.
+// There is no reason to do that.
+// Ordinary users can access audio, gpio, etc. if they are in the correct groups.
+// Giving an applications permission to do things it does not need to do
+// is a huge security risk.
 
+#ifndef __WIN32__
+	if (getuid() == 0 || geteuid() == 0) {
+	    text_color_set(DW_COLOR_ERROR);
+	    for (int n=0; n<15; n++) {
+	      dw_printf ("\n");
+	      dw_printf ("Dire Wolf requires only privileges available to ordinary users.\n");
+	      dw_printf ("Running this as root is an unnecessary security risk.\n");
+	      //SLEEP_SEC(1);
+	    }
+	}
+#endif
 
 /*
  * Default location of configuration file is current directory.
@@ -387,7 +421,7 @@ int main (int argc, char *argv[])
 
 	  /* ':' following option character means arg is required. */
 
-          c = getopt_long(argc, argv, "hP:B:gjJD:U:c:pxr:b:n:d:q:t:ul:L:Sa:E:T:e:X:A",
+          c = getopt_long(argc, argv, "hP:B:gjJD:U:c:px:r:b:n:d:q:t:ul:L:Sa:E:T:e:X:AI:i:",
                         long_options, &option_index);
           if (c == -1)
             break;
@@ -489,9 +523,43 @@ int main (int argc, char *argv[])
             }
             break;
 
-          case 'x':				/* -x for transmit calibration tones. */
+          case 'x':				/* -x N for transmit calibration tones. */
+						/* N is composed of a channel number and/or one letter */
+						/* for the mode: mark, space, alternate, ptt-only. */
 
-	    xmit_calibrate_option = 1;
+	    for (char *p = optarg; *p != '\0'; p++ ) {
+	      switch (*p) {
+	      case '0':
+	      case '1':
+	      case '2':
+	      case '3':
+	      case '4':
+	      case '5':
+	      case '6':
+	      case '7':
+	      case '8':
+	      case '9':
+	        x_opt_chan = x_opt_chan * 10 + *p - '0';
+	        if (x_opt_mode == ' ') x_opt_mode = 'a';
+	        break;
+	      case 'a':  x_opt_mode = *p; break; // Alternating tones
+	      case 'm':  x_opt_mode = *p; break; // Mark tone
+	      case 's':  x_opt_mode = *p; break; // Space tone
+	      case 'p':  x_opt_mode = *p; break; // Set PTT only
+      	      default:
+	        text_color_set(DW_COLOR_ERROR);
+	        dw_printf ("Invalid option '%c' for -x. Must be a, m, s, or p.\n", *p);
+	        text_color_set(DW_COLOR_INFO);
+      	    	exit (EXIT_FAILURE);
+      	    	break;
+      	     }
+	    }
+	    if (x_opt_chan < 0 || x_opt_chan >= MAX_CHANS) {
+	      text_color_set(DW_COLOR_ERROR);
+	      dw_printf ("Invalid channel %d for -x. \n", x_opt_chan);
+	      text_color_set(DW_COLOR_INFO);
+	      exit (EXIT_FAILURE);
+	    }
             break;
 
           case 'r':				/* -r audio samples/sec.  e.g. 44100 */
@@ -531,7 +599,7 @@ int main (int argc, char *argv[])
           case '?':
 
             /* For '?' unknown option message was already printed. */
-            usage (argv);
+            usage ();
             break;
 
 	  case 'd':				/* Set debug option. */
@@ -566,6 +634,8 @@ int main (int argc, char *argv[])
 	      case 'h':  d_h_opt++; break;			// Hamlib verbose level.
 #endif
 	      case 'x':  d_x_opt++; break;			// FX.25
+	      case '2':  d_2_opt++; break;			// IL2P
+	      case 'd':	 aprstt_debug++; break;			// APRStt (mnemonic Dtmf)
 	      default: break;
 	     }
 	    }
@@ -653,6 +723,16 @@ int main (int argc, char *argv[])
 	    X_fx25_xmit_enable = atoi(optarg);
             break;
 
+          case 'I':			// IL2P, normal polarity
+
+	    I_opt = atoi(optarg);
+            break;
+
+          case 'i':			// IL2P, inverted polarity
+
+	    i_opt = atoi(optarg);
+            break;
+
 	  case 'A':			// -A 	convert AIS to APRS object
 
 	    A_opt_ais_to_obj = 1;
@@ -663,7 +743,7 @@ int main (int argc, char *argv[])
             /* Should not be here. */
 	    text_color_set(DW_COLOR_DEBUG);
             dw_printf("?? getopt returned character code 0%o ??\n", c);
-            usage (argv);
+            usage ();
           }
 	}  /* end while(1) for options */
 
@@ -703,7 +783,7 @@ int main (int argc, char *argv[])
 	if (n_opt != 0) {
 	  audio_config.adev[0].num_channels = n_opt;
 	  if (n_opt == 2) {
-	    audio_config.achan[1].medium = MEDIUM_RADIO;
+	    audio_config.chan_medium[1] = MEDIUM_RADIO;
 	  }
 	}
 
@@ -758,7 +838,7 @@ int main (int argc, char *argv[])
 						// Will make more precise in afsk demod init.
 	    audio_config.achan[0].mark_freq = 2083;	// Actually 2083.3 - logic 1.
 	    audio_config.achan[0].space_freq = 1563;	// Actually 1562.5 - logic 0.
-	    strlcpy (audio_config.achan[0].profiles, "D", sizeof(audio_config.achan[0].profiles));
+	    strlcpy (audio_config.achan[0].profiles, "A", sizeof(audio_config.achan[0].profiles));
 	  }
 	  else {
             audio_config.achan[0].modem_type = MODEM_SCRAMBLE;
@@ -853,7 +933,45 @@ int main (int argc, char *argv[])
 
 	audio_config.recv_ber = e_recv_ber;
 
-	audio_config.fx25_xmit_enable = X_fx25_xmit_enable;
+	if (X_fx25_xmit_enable > 0) {
+	    if (I_opt != -1 || i_opt != -1) {
+	        text_color_set(DW_COLOR_ERROR);
+	        dw_printf ("Can't mix -X with -I or -i.\n");
+	        exit (EXIT_FAILURE);
+	    }
+	    audio_config.achan[0].fx25_strength = X_fx25_xmit_enable;
+	    audio_config.achan[0].layer2_xmit = LAYER2_FX25;
+	}
+
+	if (I_opt != -1 && i_opt != -1) {
+	  text_color_set(DW_COLOR_ERROR);
+	  dw_printf ("Can't use both -I and -i at the same time.\n");
+	  exit (EXIT_FAILURE);
+	}
+
+	if (I_opt >= 0) {
+	    audio_config.achan[0].layer2_xmit = LAYER2_IL2P;
+	    audio_config.achan[0].il2p_max_fec = (I_opt > 0);
+	    if (audio_config.achan[0].il2p_max_fec == 0) {
+	      text_color_set(DW_COLOR_ERROR);
+	      dw_printf ("It is highly recommended that 1, rather than 0, is used with -I for best results.\n");
+	    }
+	    audio_config.achan[0].il2p_invert_polarity = 0;	// normal
+	}
+
+	if (i_opt >= 0) {
+	    audio_config.achan[0].layer2_xmit = LAYER2_IL2P;
+	    audio_config.achan[0].il2p_max_fec = (i_opt > 0);
+	    if (audio_config.achan[0].il2p_max_fec == 0) {
+	      text_color_set(DW_COLOR_ERROR);
+	      dw_printf ("It is highly recommended that 1, rather than 0, is used with -i for best results.\n");
+	    }
+	    audio_config.achan[0].il2p_invert_polarity = 1;	// invert for transmit
+	    if (audio_config.achan[0].baud == 1200) {
+	      text_color_set(DW_COLOR_ERROR);
+	      dw_printf ("Using -i with 1200 bps is a bad idea.  Use -I instead.\n");
+	    }
+	}
 
 
 /*
@@ -870,20 +988,22 @@ int main (int argc, char *argv[])
 	  text_color_set(DW_COLOR_ERROR);
 	  dw_printf ("Pointless to continue without audio device.\n");
 	  SLEEP_SEC(5);
+	  usage ();
 	  exit (1);
 	}
 
 /*
- * Initialize the demodulator(s) and HDLC decoder.
+ * Initialize the demodulator(s) and layer 2 decoder (HDLC, IL2P).
  */
 	multi_modem_init (&audio_config);
 	fx25_init (d_x_opt);
+	il2p_init (d_2_opt);
 
 /*
  * Initialize the touch tone decoder & APRStt gateway.
  */
 	dtmf_init (&audio_config, audio_amplitude);
-	aprs_tt_init (&tt_config);
+	aprs_tt_init (&tt_config, aprstt_debug);
 	tt_user_init (&audio_config, &tt_config);
 
 /*
@@ -905,30 +1025,77 @@ int main (int argc, char *argv[])
 	xmit_init (&audio_config, d_p_opt);
 
 /*
- * If -x option specified, transmit alternating tones for transmitter
+ * If -x N option specified, transmit calibration tones for transmitter
  * audio level adjustment, up to 1 minute then quit.
- * TODO:  enhance for more than one channel.
+ * a: Alternating mark/space tones
+ * m: Mark tone (e.g. 1200Hz)
+ * s: Space tone (e.g. 2200Hz)
+ * p: Set PTT only.
+ * A leading or trailing number is the channel.
  */
 
-	if (xmit_calibrate_option) {
+	if (x_opt_mode != ' ') {
+	  if (audio_config.chan_medium[x_opt_chan] == MEDIUM_RADIO) {
+		if (audio_config.achan[x_opt_chan].mark_freq
+				&& audio_config.achan[x_opt_chan].space_freq) {
+			int max_duration = 60;
+			int n = audio_config.achan[x_opt_chan].baud * max_duration;
 
-	  int max_duration = 60;  /* seconds */
-	  int n = audio_config.achan[0].baud * max_duration;
-	  int chan = 0;
-	
-	  text_color_set(DW_COLOR_INFO);
-	  dw_printf ("\nSending transmit calibration tones.  Press control-C to terminate.\n");
+			text_color_set(DW_COLOR_INFO);
+			ptt_set(OCTYPE_PTT, x_opt_chan, 1);
 
-	  ptt_set (OCTYPE_PTT, chan, 1);
-	  while (n-- > 0) {
+			switch (x_opt_mode) {
+			default:
+			case 'a':  // Alternating tones: -x a
+				dw_printf("\nSending alternating mark/space calibration tones (%d/%dHz) on channel %d.\nPress control-C to terminate.\n",
+						audio_config.achan[x_opt_chan].mark_freq,
+						audio_config.achan[x_opt_chan].space_freq,
+						x_opt_chan);
+				while (n-- > 0) {
+					tone_gen_put_bit(x_opt_chan, n & 1);
+				}
+				break;
+			case 'm':  // "Mark" tone: -x m
+				dw_printf("\nSending mark calibration tone (%dHz) on channel %d.\nPress control-C to terminate.\n",
+						audio_config.achan[x_opt_chan].mark_freq,
+						x_opt_chan);
+				while (n-- > 0) {
+					tone_gen_put_bit(x_opt_chan, 1);
+				}
+				break;
+			case 's':  // "Space" tone: -x s
+				dw_printf("\nSending space calibration tone (%dHz) on channel %d.\nPress control-C to terminate.\n",
+						audio_config.achan[x_opt_chan].space_freq,
+						x_opt_chan);
+				while (n-- > 0) {
+					tone_gen_put_bit(x_opt_chan, 0);
+				}
+				break;
+			case 'p':  // Silence - set PTT only: -x p
+				dw_printf("\nSending silence (Set PTT only) on channel %d.\nPress control-C to terminate.\n", x_opt_chan);
+				SLEEP_SEC(max_duration);
+				break;
+			}
 
-	    tone_gen_put_bit (chan, n & 1);
+			ptt_set(OCTYPE_PTT, x_opt_chan, 0);
+			text_color_set(DW_COLOR_INFO);
+			exit(EXIT_SUCCESS);
 
-	  }
-	  ptt_set (OCTYPE_PTT, chan, 0);
-	  exit (0);
+		} else {
+			text_color_set(DW_COLOR_ERROR);
+			dw_printf("\nMark/Space frequencies not defined for channel %d. Cannot calibrate using this modem type.\n", x_opt_chan);
+			text_color_set(DW_COLOR_INFO);
+			exit(EXIT_FAILURE);
+		}
+	    } else {
+		text_color_set(DW_COLOR_ERROR);
+		dw_printf("\nChannel %d is not configured as a radio channel.\n", x_opt_chan);
+		text_color_set(DW_COLOR_INFO);
+		exit(EXIT_FAILURE);
+	    }
 	}
 
+
 /*
  * Initialize the digipeater and IGate functions.
  */
@@ -944,6 +1111,11 @@ int main (int argc, char *argv[])
 	server_init (&audio_config, &misc_config);
 	kissnet_init (&misc_config);
 
+#if (USE_AVAHI_CLIENT|USE_MACOS_DNSSD)
+	if (misc_config.kiss_port > 0 && misc_config.dns_sd_enabled)
+	  dns_sd_announce(&misc_config);
+#endif
+
 /*
  * Create a pseudo terminal and KISS TNC emulator.
  */
@@ -1008,7 +1180,7 @@ int main (int argc, char *argv[])
 
 // TODO:  Use only one printf per line so output doesn't get jumbled up with stuff from other threads.
 
-void app_process_rec_packet (int chan, int subchan, int slice, packet_t pp, alevel_t alevel, int is_fx25, retry_t retries, char *spectrum)
+void app_process_rec_packet (int chan, int subchan, int slice, packet_t pp, alevel_t alevel, fec_type_t fec_type, retry_t retries, char *spectrum)
 {	
 	
 	char stemp[500];
@@ -1017,21 +1189,31 @@ void app_process_rec_packet (int chan, int subchan, int slice, packet_t pp, alev
 	char heard[AX25_MAX_ADDR_LEN];
 	//int j;
 	int h;
-	char display_retries[32];
+	char display_retries[32];				// Extra stuff before slice indicators.
+								// Can indicate FX.25/IL2P or fix_bits.
 
-	assert (chan >= 0 && chan < MAX_CHANS);
-	assert (subchan >= -1 && subchan < MAX_SUBCHANS);
+	assert (chan >= 0 && chan < MAX_TOTAL_CHANS);		// TOTAL for virtual channels
+	assert (subchan >= -2 && subchan < MAX_SUBCHANS);
 	assert (slice >= 0 && slice < MAX_SLICERS);
 	assert (pp != NULL);	// 1.1J+
      
 	strlcpy (display_retries, "", sizeof(display_retries));
 
-	if (is_fx25) {
-	  ;
-	}
-	else if (audio_config.achan[chan].fix_bits != RETRY_NONE || audio_config.achan[chan].passall) {
-	  assert (retries >= RETRY_NONE && retries <= RETRY_MAX);
-	  snprintf (display_retries, sizeof(display_retries), " [%s] ", retry_text[(int)retries]);
+	switch (fec_type) {
+	  case fec_type_fx25:
+	    strlcpy (display_retries, " FX.25 ", sizeof(display_retries));
+	    break;
+	  case fec_type_il2p:
+	    strlcpy (display_retries, " IL2P ", sizeof(display_retries));
+	    break;
+	  case fec_type_none:
+	  default:
+	    // Possible fix_bits indication.
+	    if (audio_config.achan[chan].fix_bits != RETRY_NONE || audio_config.achan[chan].passall) {
+	      assert (retries >= RETRY_NONE && retries <= RETRY_MAX);
+	      snprintf (display_retries, sizeof(display_retries), " [%s] ", retry_text[(int)retries]);
+	    }
+	    break;
 	}
 
 	ax25_format_addrs (pp, stemp);
@@ -1056,8 +1238,11 @@ void app_process_rec_packet (int chan, int subchan, int slice, packet_t pp, alev
 	text_color_set(DW_COLOR_DEBUG);
 	dw_printf ("\n");
 
-	if (( ! q_h_opt ) && alevel.rec >= 0) {    /* suppress if "-q h" option */
+// The HEARD line.
 
+	if (( ! q_h_opt ) && alevel.rec >= 0) {    /* suppress if "-q h" option */
+// FIXME: rather than checking for ichannel, how about checking medium==radio
+	 if (chan != audio_config.igate_vchannel) {	// suppress if from ICHANNEL
 	  if (h != -1 && h != AX25_SOURCE) {
 	    dw_printf ("Digipeater ");
 	  }
@@ -1100,6 +1285,7 @@ void app_process_rec_packet (int chan, int subchan, int slice, packet_t pp, alev
 
 	    dw_printf ("%s audio level = %s  %s  %s\n", heard, alevel_text, display_retries, spectrum);
 	  }
+	 }
 	}
 
 	/* Version 1.2:   Cranking the input level way up produces 199. */
@@ -1113,6 +1299,12 @@ void app_process_rec_packet (int chan, int subchan, int slice, packet_t pp, alev
 	  text_color_set(DW_COLOR_ERROR);
 	  dw_printf ("Audio input level is too high.  Reduce so most stations are around 50.\n");
 	}
+// FIXME: rather than checking for ichannel, how about checking medium==radio
+	else if (alevel.rec < 5 && chan != audio_config.igate_vchannel) {
+
+	  text_color_set(DW_COLOR_ERROR);
+	  dw_printf ("Audio input level is too low.  Increase so most stations are around 50.\n");
+	}
 
 
 // Display non-APRS packets in a different color.
@@ -1137,6 +1329,10 @@ void app_process_rec_packet (int chan, int subchan, int slice, packet_t pp, alev
 	  text_color_set(DW_COLOR_REC);
 	  dw_printf ("[%d.dtmf%s] ", chan, ts);
 	}
+	else if (subchan == -2) {
+	  text_color_set(DW_COLOR_REC);
+	  dw_printf ("[%d.is%s] ", chan, ts);
+	}
 	else {
 	  if (ax25_is_aprs(pp)) {
 	    text_color_set(DW_COLOR_REC);
@@ -1247,7 +1443,7 @@ void app_process_rec_packet (int chan, int subchan, int slice, packet_t pp, alev
 	  // we still want to decode it for logging and other processing.
 	  // Just be quiet about errors if "-qd" is set.
 
-	  decode_aprs (&A, pp, q_d_opt);
+	  decode_aprs (&A, pp, q_d_opt, NULL);
 
 	  if ( ! q_d_opt ) {
 
@@ -1325,10 +1521,10 @@ void app_process_rec_packet (int chan, int subchan, int slice, packet_t pp, alev
 
 	flen = ax25_pack(pp, fbuf);
 
-	server_send_rec_packet (chan, pp, fbuf, flen);				// AGW net protocol
-	kissnet_send_rec_packet (chan, KISS_CMD_DATA_FRAME, fbuf, flen, -1);	// KISS TCP
-	kissserial_send_rec_packet (chan, KISS_CMD_DATA_FRAME, fbuf, flen, -1);	// KISS serial port
-	kisspt_send_rec_packet (chan, KISS_CMD_DATA_FRAME, fbuf, flen, -1);	// KISS pseudo terminal
+	server_send_rec_packet (chan, pp, fbuf, flen);					// AGW net protocol
+	kissnet_send_rec_packet (chan, KISS_CMD_DATA_FRAME, fbuf, flen, NULL, -1);	// KISS TCP
+	kissserial_send_rec_packet (chan, KISS_CMD_DATA_FRAME, fbuf, flen, NULL, -1);	// KISS serial port
+	kisspt_send_rec_packet (chan, KISS_CMD_DATA_FRAME, fbuf, flen, NULL, -1);	// KISS pseudo terminal
 
 	if (A_opt_ais_to_obj && strlen(ais_obj_packet) != 0) {
 	  packet_t ao_pp = ax25_from_text (ais_obj_packet, 1);
@@ -1337,32 +1533,52 @@ void app_process_rec_packet (int chan, int subchan, int slice, packet_t pp, alev
 	    int ao_flen = ax25_pack(ao_pp, ao_fbuf);
 
 	    server_send_rec_packet (chan, ao_pp, ao_fbuf, ao_flen);
-	    kissnet_send_rec_packet (chan, KISS_CMD_DATA_FRAME, ao_fbuf, ao_flen, -1);
-	    kissserial_send_rec_packet (chan, KISS_CMD_DATA_FRAME, ao_fbuf, ao_flen, -1);
-	    kisspt_send_rec_packet (chan, KISS_CMD_DATA_FRAME, ao_fbuf, ao_flen, -1);
+	    kissnet_send_rec_packet (chan, KISS_CMD_DATA_FRAME, ao_fbuf, ao_flen, NULL, -1);
+	    kissserial_send_rec_packet (chan, KISS_CMD_DATA_FRAME, ao_fbuf, ao_flen, NULL, -1);
+	    kisspt_send_rec_packet (chan, KISS_CMD_DATA_FRAME, ao_fbuf, ao_flen, NULL, -1);
 	    ax25_delete (ao_pp);
 	  }
 	}
 
+/*
+ * If it is from the ICHANNEL, we are done.
+ * Don't digipeat.  Don't IGate.
+ * Don't do anything with it after printing and sending to client apps.
+ */
+
+	if (chan == audio_config.igate_vchannel) {
+	    return;
+	}
+
 /* 
- * If it came from DTMF decoder, send it to APRStt gateway.
+ * If it came from DTMF decoder (subchan == -1), send it to APRStt gateway.
  * Otherwise, it is a candidate for IGate and digipeater.
  *
- * TODO: It would be useful to have some way to simulate touch tone
+ * It is also useful to have some way to simulate touch tone
  * sequences with BEACON sendto=R0 for testing.
  */
 
-	if (subchan == -1) {
+	if (subchan == -1) {		// from DTMF decoder
 	  if (tt_config.gateway_enabled && info_len >= 2) {
 	    aprs_tt_sequence (chan, (char*)(pinfo+1));
 	  }
 	}
+	else if (*pinfo == 't' && info_len >= 2 && tt_config.gateway_enabled) {
+				// For testing.
+				// Would be nice to verify it was generated locally,
+				// not received over the air.
+	  aprs_tt_sequence (chan, (char*)(pinfo+1));
+	}
 	else { 
 	
-/* Send to Internet server if option is enabled. */
-/* Consider only those with correct CRC. */
-
-	  if (ax25_is_aprs(pp) && retries == RETRY_NONE) {
+/*
+ * Send to the IGate processing.
+ * Use only those with correct CRC; We don't want to spread corrupted data!
+ * Our earlier "fix bits" hack could allow corrupted information to get thru.
+ * However, if it used FEC mode (FX.25. IL2P), we have much higher level of
+ * confidence that it is correct.
+ */
+	  if (ax25_is_aprs(pp) && ( retries == RETRY_NONE || fec_type == fec_type_fx25 || fec_type == fec_type_il2p) ) {
 
 	    igate_send_rec_packet (chan, pp);
 	  }
@@ -1377,24 +1593,23 @@ void app_process_rec_packet (int chan, int subchan, int slice, packet_t pp, alev
 
 
 /*
- * APRS digipeater.
+ * Send to APRS digipeater.
  * Use only those with correct CRC; We don't want to spread corrupted data!
+ * Our earlier "fix bits" hack could allow corrupted information to get thru.
+ * However, if it used FEC mode (FX.25. IL2P), we have much higher level of
+ * confidence that it is correct.
  */
-
-// TODO: Should also use anything received with FX.25 because it is known to be good.
-// Our earlier "fix bits" hack could allow corrupted information to get thru.
-
-	  if (ax25_is_aprs(pp) && retries == RETRY_NONE) {
+	  if (ax25_is_aprs(pp) && ( retries == RETRY_NONE || fec_type == fec_type_fx25 || fec_type == fec_type_il2p) ) {
 
 	    digipeater (chan, pp);
 	  }
 
 /*
  * Connected mode digipeater.
- * Use only those with correct CRC.
+ * Use only those with correct CRC (or using FEC.)
  */
 
-	  if (retries == RETRY_NONE) {
+	  if (retries == RETRY_NONE || fec_type == fec_type_fx25 || fec_type == fec_type_il2p) {
 
 	    cdigipeater (chan, pp);
 	  }
@@ -1469,7 +1684,9 @@ static void usage (char **argv)
 	dw_printf ("    -P xxx         Modem Profiles.\n");
 	dw_printf ("    -A             Convert AIS positions to APRS Object Reports.\n");
 	dw_printf ("    -D n           Divide audio sample rate by n for channel 0.\n");
-	dw_printf ("    -X n           1 to enable FX.25 transmit.\n");
+	dw_printf ("    -X n           1 to enable FX.25 transmit.  16, 32, 64 for specific number of check bytes.\n");
+	dw_printf ("    -I n           Enable IL2P transmit.  n=1 is recommended.  0 uses weaker FEC.\n");
+	dw_printf ("    -i n           Enable IL2P transmit, inverted polarity.  n=1 is recommended.  0 uses weaker FEC.\n");
 	dw_printf ("    -d             Debug options:\n");
 	dw_printf ("       a             a = AGWPE network protocol client.\n");
 	dw_printf ("       k             k = KISS serial port or pseudo terminal client.\n");
@@ -1487,6 +1704,8 @@ static void usage (char **argv)
 	dw_printf ("       h             h = hamlib increase verbose level.\n");
 #endif
 	dw_printf ("       x             x = FX.25 increase verbose level.\n");
+	dw_printf ("       2             2 = IL2P.\n");
+	dw_printf ("       d             d = APRStt (DTMF to APRS object translation).\n");
 	dw_printf ("    -q             Quiet (suppress output) options:\n");
 	dw_printf ("       h             h = Heard line with the audio level.\n");
 	dw_printf ("       d             d = Decoding of APRS packets.\n");
@@ -1499,6 +1718,11 @@ static void usage (char **argv)
 	dw_printf ("    -p             Enable pseudo terminal for KISS protocol.\n");
 #endif
 	dw_printf ("    -x             Send Xmit level calibration tones.\n");
+	dw_printf ("       a             a = Alternating mark/space tones.\n");
+	dw_printf ("       m             m = Steady mark tone (e.g. 1200Hz).\n");
+	dw_printf ("       s             s = Steady space tone (e.g. 2200Hz).\n");
+	dw_printf ("       p             p = Silence (Set PTT only).\n");
+	dw_printf ("        chan          Optionally add a number to specify radio channel.\n");
 	dw_printf ("    -u             Print UTF-8 test string and exit.\n");
 	dw_printf ("    -S             Print symbol tables and exit.\n");
 	dw_printf ("    -T fmt         Time stamp format for sent and received frames.\n");
@@ -1506,19 +1730,20 @@ static void usage (char **argv)
 	dw_printf ("\n");
 
 	dw_printf ("After any options, there can be a single command line argument for the source of\n");
-	dw_printf ("received audio.  This can overrides the audio input specified in the configuration file.\n");
+	dw_printf ("received audio.  This can override the audio input specified in the configuration file.\n");
 	dw_printf ("\n");
   
 #if __WIN32__
-	dw_printf ("Complete documentation can be found in the 'doc' folder\n");
+	dw_printf ("Documentation can be found in the 'doc' folder\n");
 #else
 	// TODO: Could vary by platform and build options.
-	dw_printf ("Complete documentation can be found in /usr/local/share/doc/direwolf\n");
+	dw_printf ("Documentation can be found in /usr/local/share/doc/direwolf\n");
 #endif
 	dw_printf ("or online at https://github.com/wb2osz/direwolf/tree/master/doc\n");
+	dw_printf ("additional topics: https://github.com/wb2osz/direwolf-doc\n");
+	text_color_set(DW_COLOR_INFO);
 	exit (EXIT_FAILURE);
 }
 
 
-
 /* end direwolf.c */
