@@ -1,7 +1,7 @@
 //
 //    This file is part of Dire Wolf, an amateur radio packet TNC.
 //
-//    Copyright (C) 2011, 2013, 2014, 2015, 2016, 2019  John Langner, WB2OSZ
+//    Copyright (C) 2011, 2013, 2014, 2015, 2016, 2019, 2021, 2023  John Langner, WB2OSZ
 //
 //    This program is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -56,7 +56,15 @@
  *			gen_packets -n 100 -o z2.wav
  *			atest z2.wav
  *
- *		
+ *		Variable speed. e.g. 95% to 105% of normal speed.
+ *		Required parameter is max % below and above normal.
+ *		Optionally specify step other than 0.1%.
+ *		Used to test how tolerant TNCs are to senders not
+ *		not using exactly the right baud rate.
+ *
+ *			gen_packets -v 5
+ *			gen_packets -v 5,0.5
+ *
  *------------------------------------------------------------------*/
 
 
@@ -67,6 +75,7 @@
 #include <getopt.h>
 #include <string.h>
 #include <assert.h>
+#include <math.h>
 
 #include "audio.h"
 #include "ax25_pad.h"
@@ -76,6 +85,7 @@
 #include "morse.h"
 #include "dtmf.h"
 #include "fx25.h"
+#include "il2p.h"
 
 
 /* Own random number generator so we can get */
@@ -99,6 +109,7 @@ static float g_noise_level = 0;
 static int g_morse_wpm = 0;		/* Send morse code at this speed. */
 
 
+
 static struct audio_s modem;
 
 
@@ -107,13 +118,48 @@ static void send_packet (char *str)
     	packet_t pp;
     	unsigned char fbuf[AX25_MAX_PACKET_LEN+2];
     	int flen;
-	int c;
+	int c = 0;	// channel number.
 
 	if (g_morse_wpm > 0) {
 
-	  // TODO: Why not use the destination field instead of command line option?
+	  // Why not use the destination field instead of command line option?
+	  // For one thing, this is not in TNC-2 monitor format.
 
-	  morse_send (0, str, g_morse_wpm, 100, 100);
+	  morse_send (c, str, g_morse_wpm, 100, 100);
+	}
+	else if (modem.achan[0].modem_type == MODEM_EAS) {
+
+// Generate EAS SAME signal FOR RESEARCH AND TESTING ONLY!!!
+// There could be legal consequences for sending unauhorized SAME
+// over the radio so don't do it!
+
+	  // I'm expecting to see TNC 2 monitor format.
+	  // The source and destination are ignored.
+	  // The optional destination SSID is the number of times to repeat.
+	  // The user defined data type indicator can optionally be used
+	  // for compatibility with how it is received and presented to client apps.
+	  // Examples:
+	  //	X>X-3:{DEZCZC-WXR-RWT-033019-033017-033015-033013-033011-025011-025017-033007-033005-033003-033001-025009-025027-033009+0015-1691525-KGYX/NWS-
+	  //	X>X:NNNN
+
+	  pp = ax25_from_text (str, 1);
+	  if (pp == NULL) {
+            text_color_set(DW_COLOR_ERROR);
+            dw_printf ("\"%s\" is not valid TNC2 monitoring format.\n", str);
+	    return;
+	  }
+	  unsigned char *pinfo;
+	  int info_len = ax25_get_info (pp, &pinfo);
+	  if (info_len >= 3 && strncmp((char*)pinfo, "{DE", 3) == 0) {
+	    pinfo += 3;
+	    info_len -= 3;
+	  }
+
+	  int repeat = ax25_get_ssid (pp, AX25_DESTINATION);
+	  if (repeat == 0) repeat = 1;
+
+	  eas_send (c, pinfo, repeat, 500, 500);
+	  ax25_delete (pp);
 	}
 	else {
 	  pp = ax25_from_text (str, 1);
@@ -123,6 +169,10 @@ static void send_packet (char *str)
 	    return;
 	  }
 	  flen = ax25_pack (pp, fbuf);
+	  (void)flen;
+
+	  // If stereo, put same thing in each channel.
+
 	  for (c=0; c<modem.adev[0].num_channels; c++)
 	  {
 
@@ -151,12 +201,10 @@ static void send_packet (char *str)
 	      gen_tone_put_sample (c, 0, 0);
 	    }
 #endif
-	    hdlc_send_flags (c, 8, 0);
-	    hdlc_send_flags (c, 8, 0);
-	    hdlc_send_flags (c, 8, 0);
-	    hdlc_send_flags (c, 8, 0);
-	    hdlc_send_frame (c, fbuf, flen, 0, modem.fx25_xmit_enable);
-	    hdlc_send_flags (c, 2, 1);
+
+	    layer2_preamble_postamble (c, 32, 0, &modem);
+	    layer2_send_frame (c, pp, 0, &modem);
+	    layer2_preamble_postamble (c, 2, 1, &modem);
 	  }
 	  ax25_delete (pp);
 	}
@@ -176,6 +224,12 @@ int main(int argc, char **argv)
 	int g_opt = 0;
 	int j_opt = 0;
 	int J_opt = 0;
+	int X_opt = 0;		// send FX.25
+	int I_opt = -1;		// send IL2P rather than AX.25, normal polarity
+	int i_opt = -1;		// send IL2P rather than AX.25, inverted polarity
+	double variable_speed_max_error  = 0;	// both in percent
+	double variable_speed_increment = 0.1;
+
 
 /*
  * Set up default values for the modem.
@@ -195,7 +249,7 @@ int main(int argc, char **argv)
           modem.achan[chan].baud = DEFAULT_BAUD;                        /* -b option */
 	}
 
-	modem.achan[0].medium = MEDIUM_RADIO;
+	modem.chan_medium[0] = MEDIUM_RADIO;
 
 
 /*
@@ -227,7 +281,7 @@ int main(int argc, char **argv)
 
 	  /* ':' following option character means arg is required. */
 
-          c = getopt_long(argc, argv, "gjJm:s:a:b:B:r:n:N:o:z:82M:X:",
+          c = getopt_long(argc, argv, "gjJm:s:a:b:B:r:n:N:o:z:82M:X:I:i:v:",
                         long_options, &option_index);
           if (c == -1)
             break;
@@ -266,23 +320,31 @@ int main(int argc, char **argv)
 
 						// FIXME: options should not be order dependent.
 
-              modem.achan[0].baud = atoi(optarg);
+              if (strcasecmp(optarg, "EAS") == 0) {
+	        modem.achan[0].baud = 0xEA5EA5;	// See special case below.
+	      }
+	      else {
+	        modem.achan[0].baud = atoi(optarg);
+	      }
+
               text_color_set(DW_COLOR_INFO); 
               dw_printf ("Data rate set to %d bits / second.\n", modem.achan[0].baud);
-              if (modem.achan[0].baud != 100 && (modem.achan[0].baud < MIN_BAUD || modem.achan[0].baud > MAX_BAUD)) {
-                text_color_set(DW_COLOR_ERROR);
-                dw_printf ("Use a more reasonable bit rate in range of %d - %d.\n", MIN_BAUD, MAX_BAUD);
-                exit (EXIT_FAILURE);
-              }
 
 	      /* We have similar logic in direwolf.c, config.c, gen_packets.c, and atest.c, */
 	      /* that need to be kept in sync.  Maybe it could be a common function someday. */
 
-	      if (modem.achan[0].baud == 100) {
+	      if (modem.achan[0].baud == 100) {			// What was this for?
                   modem.achan[0].modem_type = MODEM_AFSK;
                   modem.achan[0].mark_freq = 1615;
                   modem.achan[0].space_freq = 1785;
 	      }
+	      else if (modem.achan[0].baud == 0xEA5EA5) {
+		  modem.achan[0].baud = 521;			// Fine tuned later. 520.83333
+								// Proper fix is to make this float.
+                  modem.achan[0].modem_type = MODEM_EAS;
+                  modem.achan[0].mark_freq = 2083.3333;		// Ideally these should be floating point.
+                  modem.achan[0].space_freq = 1562.5000 ;
+	      }
 	      else if (modem.achan[0].baud < 600) {
                   modem.achan[0].modem_type = MODEM_AFSK;
                   modem.achan[0].mark_freq = 1600;		// Typical for HF SSB
@@ -318,6 +380,11 @@ int main(int argc, char **argv)
                   text_color_set(DW_COLOR_INFO); 
                   dw_printf ("Using scrambled baseband signal rather than AFSK.\n");
 	      }
+              if (modem.achan[0].baud != 100 && (modem.achan[0].baud < MIN_BAUD || modem.achan[0].baud > MAX_BAUD)) {
+                text_color_set(DW_COLOR_ERROR);
+                dw_printf ("Use a more reasonable bit rate in range of %d - %d.\n", MIN_BAUD, MAX_BAUD);
+                exit (EXIT_FAILURE);
+              }
               break;
 
             case 'g':				/* -g for g3ruh scrambling */
@@ -424,7 +491,7 @@ int main(int argc, char **argv)
             case '2':				/* -2 for 2 channels of sound */
   
               modem.adev[0].num_channels = 2;
-	      modem.achan[1].medium = MEDIUM_RADIO;
+	      modem.chan_medium[1] = MEDIUM_RADIO;
               text_color_set(DW_COLOR_INFO); 
               dw_printf("2 channels of sound rather than 1.\n");
               break;
@@ -453,9 +520,29 @@ int main(int argc, char **argv)
 
             case 'X':
 
-	      modem.fx25_xmit_enable = atoi(optarg);
+	      X_opt = atoi(optarg);
               break;
 
+            case 'I':			// IL2P, normal polarity
+
+	      I_opt = atoi(optarg);
+              break;
+
+            case 'i':			// IL2P, inverted polarity
+
+	      i_opt = atoi(optarg);
+              break;
+
+            case 'v':			// Variable speed data + an - this percentage
+					// optional comma and increment.
+
+	      variable_speed_max_error = fabs(atof(optarg));
+	      char *q = strchr(optarg, ',');
+	      if (q != NULL) {
+	        variable_speed_increment = fabs(atof(q+1));
+	      }
+	      break;
+
             case '?':
 
               /* Unknown option message was already printed. */
@@ -466,7 +553,7 @@ int main(int argc, char **argv)
 
               /* Should not be here. */
               text_color_set(DW_COLOR_ERROR); 
-              dw_printf("?? getopt returned character code 0%o ??\n", c);
+              dw_printf("?? getopt returned character code 0%o ??\n", (unsigned)c);
               usage (argv);
           }
 	}
@@ -507,6 +594,43 @@ int main(int argc, char **argv)
           exit (1);
 	}
 
+	if (X_opt > 0) {
+	    if (I_opt != -1 || i_opt != -1) {
+	        text_color_set(DW_COLOR_ERROR);
+	        dw_printf ("Can't mix -X with -I or -i.\n");
+	        exit (EXIT_FAILURE);
+	    }
+	    modem.achan[0].fx25_strength = X_opt;
+	    modem.achan[0].layer2_xmit = LAYER2_FX25;
+	}
+
+	if (I_opt != -1 && i_opt != -1) {
+	  text_color_set(DW_COLOR_ERROR);
+	  dw_printf ("Can't use both -I and -i at the same time.\n");
+	  exit (EXIT_FAILURE);
+	}
+
+	if (I_opt >= 0) {
+            text_color_set(DW_COLOR_INFO);
+            dw_printf ("Using IL2P normal polarity.\n");
+	    modem.achan[0].layer2_xmit = LAYER2_IL2P;
+	    modem.achan[0].il2p_max_fec = (I_opt > 0);
+	    modem.achan[0].il2p_invert_polarity = 0;	// normal
+	}
+
+	if (i_opt >= 0) {
+            text_color_set(DW_COLOR_INFO);
+            dw_printf ("Using IL2P inverted polarity.\n");
+	    modem.achan[0].layer2_xmit = LAYER2_IL2P;
+	    modem.achan[0].il2p_max_fec = (i_opt > 0);
+	    modem.achan[0].il2p_invert_polarity = 1;	// invert for transmit
+	    if (modem.achan[0].baud == 1200) {
+	      text_color_set(DW_COLOR_ERROR);
+	      dw_printf ("Using -i with 1200 bps is a bad idea.  Use -I instead.\n");
+	    }
+	}
+
+
 /*
  * Open the output file.
  */
@@ -536,6 +660,7 @@ int main(int argc, char **argv)
 	// Just use the default of minimal information.
 
 	fx25_init (1);
+	il2p_init (0);		// There are no "-d" options so far but it could be handy here.
 
         assert (modem.adev[0].bits_per_sample == 8 || modem.adev[0].bits_per_sample == 16);
         assert (modem.adev[0].num_channels == 1 || modem.adev[0].num_channels == 2);
@@ -596,9 +721,35 @@ int main(int argc, char **argv)
  */
       	text_color_set(DW_COLOR_INFO); 
       	dw_printf ("built in message...\n");
-	
 
-	if (packet_count > 0)  {
+//
+// Generate packets with variable speed.
+// This overrides any other number of packets or adding noise.
+//
+
+
+	if (variable_speed_max_error != 0) {
+
+	  int normal_speed = modem.achan[0].baud;
+
+          text_color_set(DW_COLOR_INFO);
+	  dw_printf ("Variable speed.\n");
+
+	  for (double speed_error = - variable_speed_max_error;
+			speed_error <= variable_speed_max_error + 0.001;
+			speed_error += variable_speed_increment) {
+
+	    // Baud is int so we get some roundoff.  Make it real?
+	    modem.achan[0].baud = (int)round(normal_speed * (1. + speed_error / 100.));
+	    gen_tone_init (&modem, amplitude/2, 1);
+
+	    char stemp[256];
+	    snprintf (stemp, sizeof(stemp), "WB2OSZ-15>TEST:, speed %+0.1f%%  The quick brown fox jumps over the lazy dog!", speed_error);
+	    send_packet (stemp);
+	  }
+	}	
+
+	else if (packet_count > 0)  {
 
 /*
  * Generate packets with increasing noise level.
@@ -640,14 +791,23 @@ int main(int argc, char **argv)
 	}
 	else {
 
+	  // This should send a total of 6.
+	  // Note that sticking in the user defined type {DE is optional.
+
+	  if (modem.achan[0].modem_type == MODEM_EAS) {
+	    send_packet ("X>X-3:{DEZCZC-WXR-RWT-033019-033017-033015-033013-033011-025011-025017-033007-033005-033003-033001-025009-025027-033009+0015-1691525-KGYX/NWS-");
+	    send_packet ("X>X-2:{DENNNN");
+	    send_packet ("X>X:NNNN");
+	  }
+	  else {
 /*
  * Builtin default 4 packets.
  */
-
-	  send_packet ("WB2OSZ-15>TEST:,The quick brown fox jumps over the lazy dog!  1 of 4");
-	  send_packet ("WB2OSZ-15>TEST:,The quick brown fox jumps over the lazy dog!  2 of 4");
-	  send_packet ("WB2OSZ-15>TEST:,The quick brown fox jumps over the lazy dog!  3 of 4");
-	  send_packet ("WB2OSZ-15>TEST:,The quick brown fox jumps over the lazy dog!  4 of 4");
+	    send_packet ("WB2OSZ-15>TEST:,The quick brown fox jumps over the lazy dog!  1 of 4");
+	    send_packet ("WB2OSZ-15>TEST:,The quick brown fox jumps over the lazy dog!  2 of 4");
+	    send_packet ("WB2OSZ-15>TEST:,The quick brown fox jumps over the lazy dog!  3 of 4");
+	    send_packet ("WB2OSZ-15>TEST:,The quick brown fox jumps over the lazy dog!  4 of 4");
+	  }
 	}
 
 	audio_file_close();
@@ -665,11 +825,13 @@ static void usage (char **argv)
 	dw_printf ("Options:\n");
 	dw_printf ("  -a <number>   Signal amplitude in range of 0 - 200%%.  Default 50.\n");
 	dw_printf ("  -b <number>   Bits / second for data.  Default is %d.\n", DEFAULT_BAUD);
-	dw_printf ("  -B <number>   Bits / second for data.  Proper modem selected for 300, 1200, 2400, 4800, 9600.\n");
+	dw_printf ("  -B <number>   Bits / second for data.  Proper modem selected for 300, 1200, 2400, 4800, 9600, EAS.\n");
 	dw_printf ("  -g            Scrambled baseband rather than AFSK.\n");
 	dw_printf ("  -j            2400 bps QPSK compatible with direwolf <= 1.5.\n");
 	dw_printf ("  -J            2400 bps QPSK compatible with MFJ-2400.\n");
-	dw_printf ("  -X n          Generate FX.25 frames. Specify number of check bytes: 16, 32, or 64.\n");
+	dw_printf ("  -X n           1 to enable FX.25 transmit.  16, 32, 64 for specific number of check bytes.\n");
+	dw_printf ("  -I n           Enable IL2P transmit.  n=1 is recommended.  0 uses weaker FEC.\n");
+	dw_printf ("  -i n           Enable IL2P transmit, inverted polarity.  n=1 is recommended.  0 uses weaker FEC.\n");
 	dw_printf ("  -m <number>   Mark frequency.  Default is %d.\n", DEFAULT_MARK_FREQ);
 	dw_printf ("  -s <number>   Space frequency.  Default is %d.\n", DEFAULT_SPACE_FREQ);
 	dw_printf ("  -r <number>   Audio sample Rate.  Default is %d.\n", DEFAULT_SAMPLES_PER_SEC);
@@ -677,6 +839,7 @@ static void usage (char **argv)
 	dw_printf ("  -o <file>     Send output to .wav file.\n");
 	dw_printf ("  -8            8 bit audio rather than 16.\n");
 	dw_printf ("  -2            2 channels (stereo) audio rather than one channel.\n");
+	dw_printf ("  -v max[,incr] Variable speed with specified maximum error and increment.\n");
 //	dw_printf ("  -z <number>   Number of leading zero bits before frame.\n");
 //	dw_printf ("                  Default is 12 which is .01 seconds at 1200 bits/sec.\n");
 
@@ -685,6 +848,7 @@ static void usage (char **argv)
 	dw_printf ("the default built-in message. The format should correspond to\n");
 	dw_printf ("the standard packet monitoring representation such as,\n\n");
 	dw_printf ("    WB2OSZ-1>APDW12,WIDE2-2:!4237.14NS07120.83W#\n");
+	dw_printf ("User defined content can't be used with -n option.\n");
 	dw_printf ("\n");
 	dw_printf ("Example:  gen_packets -o x.wav \n");
 	dw_printf ("\n");
