@@ -46,6 +46,7 @@
 #include "demod_9600.h"		/* for descramble() */
 #include "ptt.h"
 #include "fx25.h"
+#include "il2p.h"
 
 
 //#define TEST 1				/* Define for unit testing. */
@@ -151,7 +152,7 @@ void hdlc_rec_init (struct audio_s *pa)
 	for (ch = 0; ch < MAX_CHANS; ch++)
 	{
 
-	  if (pa->achan[ch].medium == MEDIUM_RADIO) {
+	  if (pa->chan_medium[ch] == MEDIUM_RADIO) {
 
 	    num_subchan[ch] = pa->achan[ch].num_subchan;
 
@@ -496,6 +497,7 @@ void hdlc_rec_bit (int chan, int subchan, int slice, int raw, int is_scrambled,
 
 	if (g_audio_p->achan[chan].modem_type != MODEM_AIS) {
 	  fx25_rec_bit (chan, subchan, slice, dbit);
+	  il2p_rec_bit (chan, subchan, slice, raw);	// Note: skip NRZI.
 	}
 
 /*
@@ -759,7 +761,7 @@ void dcd_change (int chan, int subchan, int slice, int state)
  *
  * Name:        hdlc_rec_data_detect_any
  *
- * Purpose:     Determine if the radio channel is curently busy
+ * Purpose:     Determine if the radio channel is currently busy
  *		with packet data.
  *		This version doesn't care about voice or other sounds.
  *		This is used by the transmit logic to transmit only
@@ -774,7 +776,7 @@ void dcd_change (int chan, int subchan, int slice, int state)
  * Description:	We have two different versions here.
  *
  *		hdlc_rec_data_detect_any sees if ANY of the decoders
- *		for this channel are receving a signal.   This is
+ *		for this channel are receiving a signal.   This is
  *		used to determine whether the channel is clear and
  *		we can transmit.  This would apply to the 300 baud
  *		HF SSB case where we have multiple decoders running
